# -*- coding: utf-8 -*-

import json
import socket
from logging.handlers import DatagramHandler

from .. import constants


class LogHandler(DatagramHandler):
    source = None
    source_ip = None
    role_list = None

    def __init__(self, host=None, port=None, source=None, role_list=None):
        DatagramHandler.__init__(self,
                                 host or constants.HOST,
                                 port or constants.PORT)

        # source_ip
        cls = self.__class__
        if not cls.source_ip:
            try:
                # 一步步来，这样第二步报错时，起码也把名字设置上了
                cls.source_ip = socket.gethostname()
                # 有些电脑上会很慢，还是不用了，就用名字算了
                # cls.source_ip = socket.gethostbyname(cls.source_ip)
            except:
                pass

            cls.source_ip = cls.source_ip or 'none'

        self.source = source
        self.role_list = role_list

    def emit(self, record):
        source = '%s@%s' % (self.source, self.__class__.source_ip)
        content = self.format(record)
        try:
            s = json.dumps(dict(source=source, role_list=self.role_list, content=content))
            # python3的json.dumps返回的是str
            if not isinstance(s, bytes):
                s = s.encode('utf-8')
            self.send(s)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def makeSocket(self):
        return DatagramHandler.makeSocket(self)



# -*- coding: utf-8 -*-

import sys
from .six import reraise


def import_string(import_name, silent=False):
    """Imports an object based on a string.  This is useful if you want to
    use import paths as endpoints or something similar.  An import path can
    be specified either in dotted notation (``xml.sax.saxutils.escape``)
    or with a colon as object delimiter (``xml.sax.saxutils:escape``).

    If `silent` is True the return value will be `None` if the import fails.

    :param import_name: the dotted name for the object to import.
    :param silent: if set to `True` import errors are ignored and
                   `None` is returned instead.
    :return: imported object
    """
    # force the import name to automatically convert to strings
    # __import__ is not able to handle unicode strings in the fromlist
    # if the module is a package
    import_name = str(import_name).replace(':', '.')
    try:
        try:
            __import__(import_name)
        except ImportError:
            if '.' not in import_name:
                raise
        else:
            return sys.modules[import_name]

        module_name, obj_name = import_name.rsplit('.', 1)
        try:
            module = __import__(module_name, None, None, [obj_name])
        except ImportError:
            # support importing modules not yet set up by the parent module
            # (or package for that matter)
            module = import_string(module_name)

        try:
            return getattr(module, obj_name)
        except AttributeError as e:
            raise ImportError(e)

    except ImportError as e:
        if not silent:
            t, v, tb = sys.exc_info()
            reraise(t, v, tb)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
用来作为接受log传递的agent
通过udp通道。

不使用 gevent，因为测试了 gevent 对 smtplib 似乎没法异步，还是会阻塞
"""

from __future__ import unicode_literals

import logging.config
import json
from collections import defaultdict

from .six import socketserver, _thread
from .utils import import_string
from .log import logger


class Server(object):
    backend_dict = None

    def __init__(self, config=None):
        self.config = config
        if hasattr(config, 'LOGGING'):
            logging.config.dictConfig(config.LOGGING)

        self.backend_dict = dict()
        for name, backend_conf in self.config.BACKENDS.items():
            _class = import_string(backend_conf['class'])
            backend = _class(**backend_conf['init_data'])

            self.backend_dict[name] = backend

    def handle_message(self, message, address):
        recv_dict = json.loads(message)

        title = '[%s]Attention!' % recv_dict.get('source')
        content = recv_dict.get('content')

        logger.info('%s\n%s', title, content)

        role_list = recv_dict.get('role_list') or ('default',)

        # backend_name -> params
        merged_backends = defaultdict(dict)

        for role in role_list:
            handler_list = self.config.ROLES.get(role)
            if handler_list is None:
                continue

            for handler in handler_list:
                backend_name = handler['backend']
                params = handler['params']

                merged_backends[backend_name] = self._merge_backend_params(
                    merged_backends[backend_name],
                    params
                )

        for backend_name, params in merged_backends.items():
            _thread.start_new_thread(self._process_backend_emit, (backend_name, params, title, content))

    def _merge_backend_params(self, params1, params2):
        """
        因为有可能不同role之间对应同样的发送人，为了避免重复，所以需要合并
        """

        merged_keys = set(params1.keys()) | set(params2.keys())
        merged_params = dict()

        for key in merged_keys:
            value1 = params1.get(key) or list()
            value2 = params2.get(key) or list()

            merged_params[key] = list(set(value1) | set(value2))

        return merged_params

    def _process_backend_emit(self, backend_name, params, title, content):
        """
        为了支持单独线程，所以独立出来
        """
        backend = self.backend_dict[backend_name]

        try:
            if not backend.emit(title, content, **params):
                logger.error('emit error. backend: %s, params: %s, title: %s, content: %s',
                             backend_name, params, title, content)
        except:
            logger.error('exc occur. backend: %s, params: %s, title: %s, content: %s',
                         backend_name, params, title, content, exc_info=True)

    def run(self, host, port):
        class ThreadedUDPRequestHandler(socketserver.BaseRequestHandler):
            def handle(sub_self):
                message = sub_self.request[0]
                try:
                    self.handle_message(message, sub_self.client_address)
                except:
                    logger.error('exc occur. message: %r, address: %s', message, sub_self.client_address, exc_info=True)

        class MyUDPServer(socketserver.ThreadingUDPServer):
            daemon_threads = True
            allow_reuse_address = True

        logger.info('running on %s:%s', host, port)

        server = MyUDPServer((host, port), ThreadedUDPRequestHandler)

        try:
            server.serve_forever()
        except KeyboardInterrupt:
            pass
        except:
            logger.error('exc occur.', exc_info=True)



# -*- coding: utf-8 -*-

import socket
import json
from .. import constants


class Client(object):

    sock = None

    def __init__(self, host=None, port=None):
        self.host = host or constants.HOST
        self.port = port or constants.PORT

        # 一开始就创建好sock
        self._create_socket()

    def send(self, source, content, role_list=None):
        """
        发送
        """
        s = json.dumps(dict(source=source, role_list=role_list, content=content))
        # python3的json.dumps返回的是str
        if not isinstance(s, bytes):
            s = s.encode('utf-8')
        self.sock.sendto(s, (self.host, self.port))

    def _create_socket(self):
        """
        创建socket
        """
        # 先尝试关闭掉
        self.close()

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    def close(self):
        """
        关闭
        """
        if self.sock is None:
            return

        try:
            self.sock.close()
        except:
            pass
        finally:
            self.sock = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
启动 flylog_agent
"""

import types
import argparse
import sys

import flylog
from flylog import Server, constants


def build_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--host', default=constants.HOST, help='bind host', action='store')
    parser.add_argument('-p', '--port', default=constants.PORT, type=int, help='bind port', action='store')
    parser.add_argument('-c', '--config', help='config file', action='store', required=True)
    parser.add_argument('-v', '--version', action='version', version='%s' % flylog.__version__)
    return parser


def load_config(filename):
    d = types.ModuleType('config')
    d.__file__ = filename
    try:
        with open(filename) as config_file:
            exec(compile(config_file.read(), filename, 'exec'), d.__dict__)
    except IOError as e:
        e.strerror = 'Unable to load configuration file (%s)' % e.strerror
        raise
    return d


def main():

    args = build_parser().parse_args()

    app = Server(config=load_config(args.config))

    try:
        app.run(args.host, args.port)
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name="flylog",
    version='0.2.6',
    zip_safe=False,
    platforms='any',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    install_requires=['requests'],
    scripts=['flylog/bin/run_flylog.py'],
    url="https://github.com/dantezhu/flylog",
    license="BSD",
    author="dantezhu",
    author_email="zny2008@gmail.com",
    description="make log fly to mail or other",
)



# -*- coding: utf-8 -*-

import copy
import random

from ..log import logger


class MailBackend(object):
    """
    发送邮件
    """

    def __init__(self, sender_list):
        """
        初始化
        sender_list可以保证只要发送失败就尝试下一个
        """
        self.sender_list = sender_list

    def emit(self, title, content, receiver_list):
        """
        发送
        """

        sender_list = copy.deepcopy(self.sender_list)

        while sender_list:
            random.shuffle(sender_list)

            # 取出最后一个
            params = sender_list.pop()

            try:
                self._sendmail(params['host'], params['port'], params['sender'],
                               receiver_list, title, content,
                               username=params['username'], password=params['password'],
                               use_ssl=params['use_ssl'], use_tls=params['use_tls'],
                               )
                return True
            except:
                logger.error('exc occur. params: %s', params, exc_info=True)
        else:
            # 就是循环完了，也没发送成功
            return False

    def _sendmail(self, host, port, sender, receiver_list, subject, content,
                  content_type='plain', encoding='utf-8',
                  username=None, password=None, use_ssl=False, use_tls=False, debuglevel=0
                  ):
        """
        发送邮件
        content_type: plain / html
        """
        import smtplib
        from email.mime.text import MIMEText
        from email.header import Header
        from email.utils import formatdate

        mail_msg = MIMEText(content, content_type, encoding)
        mail_msg['Subject'] = Header(subject, encoding)
        mail_msg['From'] = sender
        mail_msg['To'] = ', '.join(receiver_list)
        mail_msg['Date'] = formatdate()

        if use_ssl:
            mail_client = smtplib.SMTP_SSL(host, port)
        else:
            mail_client = smtplib.SMTP(host, port)

        mail_client.set_debuglevel(debuglevel)

        if use_tls:
            mail_client.starttls()

        if username and password:
            mail_client.login(username, password)

        # 发邮件
        mail_client.sendmail(sender, receiver_list, mail_msg.as_string())
        mail_client.quit()
